@@ -34,4 +34,4 @@ class LinkIcon(Element):
         self.attrs = [('href', self.url)]
         if self.target:
             self.attrs.append(('target', self.target))
-        self.append_inner(Icon(css_class))
\ No newline at end of file
+        self.append(Icon(css_class))
\ No newline at end of file
